import os
import time
import math
import json
import math
import asyncio
import traceback
import pandas as pd
from .constants import ADD_DEVICES, PASSED
from .logger import logger

def add_device_using_mac_serial(central_api_req, input_data : dict, **kwargs) -> dict:
    """
    Adding device to the Central Account

        Parameters:
            central_api_req (CentralAPIRequest) : Request object to make HTTP call
            input_data (json): Input data(mac and serial) to add devices
            **kwargs: Additional or Optional arguments

        Returns:
            dict: includes status_code, input(list(dict)) and ouput(str)
    """
    try:
        input_dict = []
        for each_input_dict in input_data:
            if ('mac' and 'serial') in each_input_dict.keys():
                input_dict.append(each_input_dict)
            else:
                raise Exception("Invalid request data")
        input_data = input_dict
        pd_device_details = pd.DataFrame(input_data)
        limit = 20
        # add_device_output_list = []
        if len(pd_device_details) > 0:
            serial_list = pd_device_details.get("mac").to_list()
            mac_list = pd_device_details.get("serial").to_list()
            device_details = [{"mac": i[0], "serial": i[1]} for i in zip(mac_list, serial_list)]
            for index in range(math.ceil(len(device_details) / limit)):
                subtasks = []
                for device_info in device_details[index * limit:(index + 1) * limit]:
                    subtasks.append(add_device(central_api_req, device_info))
                if os.name == "nt":
                    asyncio.set_event_loop_policy(asyncio.WindowsSelectorEventLoopPolicy())
                asyncio.run(synctask(subtasks))
                if len(device_details[index * limit:(index + 1) * limit]) > 20:
                    logger.debug("Time sleep started")
                    time.sleep(20)
                    logger.debug("Time sleep ended")
            # return add_device_output_list
        else:
            error_message = "0 device selected to perform the task"
            logger.error(error_message)
            return [
                {"status_code": None, "input": input_data, "output": error_message}
            ]

    except Exception as e:
        logger.error(e)
        error_message = "Error adding the device to central inventory"
        logger.error(error_message)
        return [
            {"status_code": None, "input": input_data, "output": error_message}
        ]


async def add_device(central_api_req, device_info):
    """
    Asynchronous function to add device to central

        Parameters:
                central_api_req (CentralAPIRequest) : Request object to make HTTP call
                device_info (dictionary): single row from the input file

        Returns:
                None
    """
    try:
        method = "POST"
        api_path = ADD_DEVICES
        resp, resp_data = await central_api_req.asyncrequest(method, api_path, data=json.dumps([device_info]))
        if resp.status == 200:
            error_message = f"{device_info.get('serial')} - Device added"
            logger.info(f"{device_info.get('serial')} - Device added")
            return {"status_code": resp.status, "input": [device_info], "output": error_message}
        elif resp.status == 429:
            error_message = f"Central API error occured while adding {device_info.get('serial')}"
            logger.error(error_message)
            return {"status_code": resp.status, "input": [device_info], "output": error_message}
            
        elif resp.status == 500:
            error_message = f"{device_info.get('serial')} Failed to add IAP"
            logger.error(error_message)
            return {"status_code": resp.status, "input": [device_info], "output": error_message}
            
        # elif resp_data.get("code") == "ATHENA_ERROR_NO_DEVICE" and resp_data.get("extra").get("message").get("invalid_device") != []:
        #     if resp_data.get("extra").get("message").get("invalid_device")[0].get("status") == "ATHENA_ERROR_DEVICE_ALREADY_EXIST":
        #         error_message = "Device already added"
        #         logger.info("Device already added")
        #         return {"status_code": resp.status, "input": [device_info], "output": error_message}
                
        else:
            error_message = f"{device_info.get('serial')} Failed to add IAP"
            logger.error(error_message)
            return {"status_code": resp.status, "input": [device_info], "output": error_message}
            
    except Exception as e:
        logger.error(e)
        error_message = f"Error in adding AP to inventory"
        logger.error(error_message)
        return {"status_code": resp.status, "input": [device_info], "output": error_message}
        
async def synctask(subtasks):
	await asyncio.gather(*subtasks)
